@@ -49,6 +49,7 @@
 #include <QTextDocument> // for Qt::escape
 #include <QLocale>
 #include <QIcon>
+#include <QColor>
 
 #include <gpgme++/gpgmepp_version.h>
 #if GPGMEPP_VERSION > 0x10900
@@ -236,11 +237,11 @@ static QString protect_whitespace(QString s)
 template <typename T_arg>
 QString format_row(const QString &field, const T_arg &arg)
 {
-    return i18n("<tr><th>%1:</th><td>%2</td></tr>", protect_whitespace(field), arg);
+    return QStringLiteral("<tr><th>%1:</th><td>%2</td></tr>").arg(protect_whitespace(field), arg);
 }
 QString format_row(const QString &field, const QString &arg)
 {
-    return i18n("<tr><th>%1:</th><td>%2</td></tr>", protect_whitespace(field), arg.toHtmlEscaped());
+    return QStringLiteral("<tr><th>%1:</th><td>%2</td></tr>").arg(protect_whitespace(field), arg.toHtmlEscaped());
 }
 QString format_row(const QString &field, const char *arg)
 {
@@ -271,7 +272,7 @@ QString format_keyusage(const Key &key)
         capabilities.push_back(i18n("Encryption"));
     }
     if (key.canCertify()) {
-        capabilities.push_back(i18n("Certifying Identities"));
+        capabilities.push_back(i18n("Certifying User-IDs"));
     }
     if (key.canAuthenticate()) {
         capabilities.push_back(i18n("SSH Authentication"));
@@ -318,13 +319,13 @@ QString Formatting::toolTip(const Key &key, int flags)
                     }
                 }
                 if (fullyTrusted == key.numUserIDs()) {
-                    result = i18n("All identities are certified.");
+                    result = i18n("All User-IDs are certified.");
                     const auto compliance = complianceStringForKey(key);
                     if (!compliance.isEmpty()) {
                         result += QStringLiteral("<br>") + compliance;
                     }
                 } else {
-                    result = i18np("One identity is not certified.", "%1 identities are not certified.", key.numUserIDs() - fullyTrusted);
+                    result = i18np("One User-ID is not certified.", "%1 User-IDs are not certified.", key.numUserIDs() - fullyTrusted);
                 }
             }
         else {
@@ -379,7 +380,7 @@ QString Formatting::toolTip(const Key &key, int flags)
     }
     if (flags & OwnerTrust) {
         if (key.protocol() == OpenPGP) {
-            result += format_row(i18n("Ownertrust"), ownerTrustShort(key));
+            result += format_row(i18n("Certification trust"), ownerTrustShort(key));
         } else if (key.isRoot()) {
             result += format_row(i18n("Trusted issuer?"),
                                  key.userID(0).validity() == UserID::Ultimate ? i18n("Yes") :
@@ -587,26 +588,65 @@ QString Formatting::validityShort(const UserID::Signature &sig)
     switch (sig.status()) {
     case UserID::Signature::NoError:
         if (!sig.isInvalid()) {
-            if (sig.certClass() > 0) {
+            /* See RFC 4880 Section 5.2.1 */
+            switch (sig.certClass()) {
+            case 0x10: /* Generic */
+            case 0x11: /* Persona */
+            case 0x12: /* Casual */
+            case 0x13: /* Positive */
+                return i18n("valid");
+            case 0x30:
+                return i18n("revoked");
+            default:
                 return i18n("class %1", sig.certClass());
-            } else {
-                return i18nc("good/valid signature", "good");
             }
         }
-#if QT_VERSION >= QT_VERSION_CHECK(5,8,0)
         Q_FALLTHROUGH();
-#endif
-    // fall through:
+        // fall through:
     case UserID::Signature::GeneralError:
         return i18n("invalid");
     case UserID::Signature::SigExpired:   return i18n("expired");
     case UserID::Signature::KeyExpired:   return i18n("certificate expired");
     case UserID::Signature::BadSignature: return i18nc("fake/invalid signature", "bad");
-    case UserID::Signature::NoPublicKey:  return QString();
+    case UserID::Signature::NoPublicKey:  return i18n("no public key");
     }
     return QString();
 }
 
+QIcon Formatting::validityIcon(const UserID::Signature &sig)
+{
+    switch (sig.status()) {
+    case UserID::Signature::NoError:
+        if (!sig.isInvalid()) {
+            /* See RFC 4880 Section 5.2.1 */
+            switch (sig.certClass()) {
+            case 0x10: /* Generic */
+            case 0x11: /* Persona */
+            case 0x12: /* Casual */
+            case 0x13: /* Positive */
+                return QIcon::fromTheme(QStringLiteral("emblem-success"));
+            case 0x30:
+                return QIcon::fromTheme(QStringLiteral("emblem-error"));
+            default:
+                return QIcon();
+            }
+        }
+        Q_FALLTHROUGH();
+        // fall through:
+    case UserID::Signature::BadSignature:
+    case UserID::Signature::GeneralError:
+        return QIcon::fromTheme(QStringLiteral("emblem-error"));
+    case UserID::Signature::SigExpired:
+    case UserID::Signature::KeyExpired:
+        return QIcon::fromTheme(QStringLiteral("emblem-information"));
+    case UserID::Signature::NoPublicKey:
+        return QIcon::fromTheme(QStringLiteral("emblem-question"));
+    default:
+        // should not happen
+        return QIcon();
+    }
+}
+
 QString Formatting::formatKeyLink(const Key &key)
 {
     if (key.isNull()) {
@@ -871,11 +911,11 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
     // for now we only have DE-VS
     if (complianceMode() == QStringLiteral("de-vs")) {
         if (uidsHaveFullValidity(key) && isKeyDeVs(key)) {
-            return i18nc("VS-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant with that..",
-                         "May be used for VS-compliant communication.");
+            return i18nc("VS-NfD conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant with that..",
+                         "May be used for VS-NfD-compliant communication.");
         } else {
-            return i18nc("VS-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant to that..",
-                         "May <b>not</b> be used for VS-compliant communication.");
+            return i18nc("VS-NfD-conforming is a German standard for restricted documents. For which special restrictions about algorithms apply. The string describes if a key is compliant to that..",
+                         "May <b>not</b> be used for VS-NfD-compliant communication.");
         }
     }
     return QString();
@@ -887,8 +927,8 @@ QString Formatting::complianceStringShort(const GpgME::Key &key)
         if (complianceMode() == QStringLiteral("de-vs")
             && Formatting::isKeyDeVs(key)) {
             return QStringLiteral("★ ") +
-                i18nc("VS-conforming is a German standard for restricted documents for which special restrictions about algorithms apply.  The string states that a key is compliant with that.",
-                      "VS-compliant");
+                i18nc("VS-NfD-conforming is a German standard for restricted documents for which special restrictions about algorithms apply.  The string states that a key is compliant with that.",
+                      "VS-NfD-compliant");
         }
         return i18nc("As in all user IDs are valid.", "certified");
     }
